

#include <MozziGuts.h>
#include <Oscil.h> // oscillator 
#include <tables/cos2048_int8.h> // table for Oscils to play
#include <Smooth.h>
#include <AutoMap.h> // maps unpredictable inputs to a range
 
// desired carrier frequency max and min, for AutoMap
const int MIN_CARRIER_FREQ = 1200;
const int MAX_CARRIER_FREQ = 10;

// desired intensity max and min, for AutoMap, note they're inverted for reverse dynamics
const int MIN_INTENSITY = 1000;
const int MAX_INTENSITY2 = 1;

// desired mod speed max and min, for AutoMap, note they're inverted for reverse dynamics
const int MIN_MOD_SPEED = 20000;
const int MAX_MOD_SPEED = 10;

AutoMap kMapCarrierFreq(0,1023,MAX_CARRIER_FREQ, MIN_CARRIER_FREQ);
AutoMap kMapIntensity(0,1023,MAX_INTENSITY2,MIN_INTENSITY);
AutoMap kMapModSpeed(0,1023,MIN_MOD_SPEED,MAX_MOD_SPEED);


///pins and things

float gamTone = 0; 
int trigPin = 4;
int triggerIn = digitalRead(trigPin); //tigger
int offBeatPin = 2;
int offBeatSwitch = digitalRead(offBeatPin);//octave switch
int pitchPin = A3;
int pitchFreq = analogRead(pitchPin);//pitch
int pitchFreq2 = analogRead(pitchPin);//pitch
float gamArray[8]{204.40, 408.81, 433.51, 597.81, 648.19, 820.33,872.61,1198.57};
//int gamArray[10]{204.40, 408.81, 433.51, 597.81, 648.19, 820.33,872.61,1198.57};
int pulseLenPin = A2;
int latch = 0;
   int Soundpin = 9;
const int ledPin =  2;// the number of the LED pin
int tempoPin = A1;
const int KNOB_PIN = A3; //PITCH set the input for the knob
const int LDR1_PIN = A2; // INTENSITYset the analog input for fm_intensity
const int LDR2_PIN = A0; // RATE set the analog input for mod rate
int modePin = 3; 
int modeSwitch = LOW; //int testD10 = digitalRead(10);
int volumeCont = 1; //volume control
int triggerPin = 4;
// Variables will change :
int ledState = LOW;             // ledState used to set the LED

// Generally, you should use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long previousMillis = millis();  
unsigned long previousMillis2  = millis(); // will store last time LED was updated

// constants won't change :
unsigned long interval = 1000;


Oscil<COS2048_NUM_CELLS, AUDIO_RATE> aCarrier(COS2048_DATA);
Oscil<COS2048_NUM_CELLS, AUDIO_RATE> aModulator(COS2048_DATA);
Oscil<COS2048_NUM_CELLS, CONTROL_RATE> kIntensityMod(COS2048_DATA);

int mod_ratio = 5; // brightness (harmonics)
long fm_intensity; // carries control info from updateControl to updateAudio

// smoothing for intensity to remove clicks on transitions
float smoothness = 0.95f;
Smooth <long> aSmoothIntensity(smoothness);


void setup(){
//Serial.begin(9600); // for Teensy 3.1, beware printout can cause glitches
 // Serial.begin(115200); // set up the Serial output so we can look at the piezo values // set up the Serial output so we can look at the light level
  startMozzi(); // :))
  //pinMode(triggerPin, OUTPUT);
}


void updateControl(){

interval = mozziAnalogRead(tempoPin);
  //volumeCont = 1;
  // read the knob
  int knob_value = mozziAnalogRead(KNOB_PIN); // value is 0-1023
  
  // map the knob to carrier frequency
  int carrier_freq = kMapCarrierFreq(knob_value);
  int mod_freq =0;

  //previous settings for mod
mod_freq = carrier_freq * mod_ratio;
 // }

  
  // set the FM oscillator frequencies
  //VOLUME CONTROL / DIGITAL INPUT X FREQ BY EITHER 1 OR 0 TO MUTE THE SOUND
    aCarrier.setFreq((carrier_freq)* volumeCont);
  //aCarrier.setFreq(carrier_freq); 
  aModulator.setFreq(mod_freq);
  
  // read the light dependent resistor on the width Analog input pin
  int LDR1_value= mozziAnalogRead(LDR1_PIN); // value is 0-1023
  

  int LDR1_calibrated = kMapIntensity(LDR1_value);
  
  
 // calculate the fm_intensity
  fm_intensity = ((long)LDR1_calibrated * (kIntensityMod.next()+128))>>8; // shift back to range after 8 bit multiply
 
  
  // read the light dependent resistor on the speed Analog input pin
  int LDR2_value= mozziAnalogRead(LDR2_PIN); // value is 0-1023
 
  
  // use a float here for low frequencies
float mod_speed = (float)kMapModSpeed(LDR2_value)/1000;
  //float mod_speed = 1000;
 
  kIntensityMod.setFreq(mod_speed);
  }


int updateAudio(){
long modulation = (aSmoothIntensity.next(fm_intensity) * aModulator.next()) * volumeCont;
return aCarrier.phMod(modulation);
}

void tonePlay(){

int pulseLen = analogRead(pulseLenPin);//pulseLenght
pulseLen = map(pulseLen,1200, 0, 1000, 0);
pitchFreq = analogRead(pitchPin);//pitch
pitchFreq2 = map(pitchFreq,1200, 0, 8, 0);
gamTone = gamArray[pitchFreq2];
triggerIn = digitalRead(trigPin); //tigger

offBeatSwitch = digitalRead(offBeatPin);//octave switch
  if(offBeatSwitch == HIGH){
//TONE GENERATOR HERE
if(triggerIn == LOW){        // mute tone
 //gamTone = gamArray[random(0,7)];
//tone(Soundpin, (gamTone - pitchFreq));
tone(Soundpin, pitchFreq);
delay(pulseLen);
noTone(Soundpin);
latch = 1;
}
if(triggerIn == HIGH){        // mute tone
noTone(Soundpin);
latch = 0;
}
}
////////////
  if(offBeatSwitch == LOW){
//TONE GENERATOR HERE
if(triggerIn == HIGH and latch == 0){        // mute tone
 //gamTone = gamArray[random(0,7)];
tone(Soundpin, gamTone);
latch =1;
//tone(Soundpin, (pitchFreq /2));
delay(pulseLen);
noTone(Soundpin);
}
if(triggerIn == LOW){        // mute tone
noTone(Soundpin);
latch = 0;
}
}
delay(1);

}
void loop(){

  audioHook();
 offBeatSwitch = digitalRead(offBeatPin);//octave switch
if(offBeatSwitch == LOW){
 volumeCont = 0;
 
 triggerIn = digitalRead(trigPin);
    if(triggerIn == LOW){
      volumeCont = 0;
     }
     if(triggerIn == HIGH){
      volumeCont = 1;
     }
}

if(offBeatSwitch == HIGH){
 volumeCont = 1;}

}



